import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  MetadataKeyType,
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';


export interface InjectAnnotationAttributes {
  value: string[];
}

export class InjectAnnotation implements Annotation {
  public readonly attributes: InjectAnnotationAttributes;

  public constructor(value: string | string[] | InjectAnnotationAttributes) {

    if (undefined === value) {
      throw new Error(`value attribute must be specified.`);
    }

    if (TypeUtil.isString(value)) {
      this.attributes = {
        value: [<string>value],
      };
    } else if (TypeUtil.isArray(value)) {
      this.attributes = {
        value: <string[]>value,
      };
    } else {
      this.attributes = <InjectAnnotationAttributes>value;
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('Inject');
  }

  public propertyDecorator?(target: Object, propertyKey: PropertyKeyType): void {
    console.log('Inject');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('Inject');
  }

  public parameterDecorator?(target: Object, propertyKey: PropertyKeyType, parameterIndex: number): void {
    console.log('Inject');
  }

}

export const Inject = Decorator.create(InjectAnnotation);
