import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  MetadataKeyType,
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';


export interface ResourceAnnotationAttributes {
  value: string[];
}

export class ResourceAnnotation implements Annotation {
  public readonly attributes: ResourceAnnotationAttributes;

  public constructor(value: string | string[] | ResourceAnnotationAttributes) {

    if (undefined === value) {
      throw new Error(`value attribute must be specified.`);
    }

    if (TypeUtil.isString(value)) {
      this.attributes = {
        value: [<string>value],
      };
    } else if (TypeUtil.isArray(value)) {
      this.attributes = {
        value: <string[]>value,
      };
    } else {
      this.attributes = <ResourceAnnotationAttributes>value;
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('Resource');
  }

  public propertyDecorator?(target: Object, propertyKey: PropertyKeyType): void {
    console.log('Resource');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('Resource');
  }

  public parameterDecorator?(target: Object, propertyKey: PropertyKeyType, parameterIndex: number): void {
    console.log('Resource');
  }
}

export const Resource = Decorator.create(ResourceAnnotation);
